var HB = HB || {};

/**
 * editor
 * 编辑器组件
 *
 *     作者: 聂心原 (joshua@huoban.com)
 * 创建时间: 2012-07-25 18:36:35
 * 修改记录:
 *
 * $Id: editor.js 3750 2013-06-30 05:16:46Z joshua $
 */
(function() {

    var _editor = {};

    /**
     * getInstance
     * 获取编辑器实例
     *
     * @param  string elementId 元素Id
     * @return object
     */
    _editor.getInstance = function(elementId) {
        return new UEditor(elementId);
    };

    /**
     * UEditor
     * 封装过的百度编辑器构造方法
     *
     * @param string elementId 元素ID
     */
    var UEditor = function(elementId) {

        var _instance = UE.getEditor(elementId);

        /**
         * destroy
         * 销毁编辑器对象
         *
         * @return void
         */
        this.destroy = function() {
            _instance = UE.delEditor(elementId);
        };

        /**
         * on
         * 注册事件
         *
         * @param string  eventType 事件类型
         * @param function callback  回调方法
         * @return void
         */
        this.on = function(eventType, callback) {
            return _instance.addListener(eventType, callback);
        };

        /**
         * exec
         * 执行命令
         *
         * @param string cmd 命令标识
         *   + horizontal 分割线
         *   + insertunorderedlist 无序列表
         *   + insertorderedlist 有序列表
         *   + underline 下划线
         *   + italic 斜体
         *   + bold 加粗
         * @param mixed arguments 命令参数
         *   ...
         * @return void
         */
        this.exec = function(cmd) {
            return _instance.execCommand.apply(_instance, arguments);
        };

        /**
         * getContent
         * 获取用户输入内容
         *
         * @return string
         */
        this.getContent = function() {
            return _instance.getContent();
        };

        /**
         * setContent
         * 设置内容
         *
         * @param string html html内容
         */
        this.setContent = function(html) {
            return _instance.setContent(html);
        };

        /**
         * reset
         * 重置编辑器(清除内容、清空回退列表)
         *
         * @return void
         */
        this.reset = function() {
            return _instance.reset();
        };

        /**
         * focus
         * 让编辑器获得焦点，toEnd确定focus位置
         *
         * @param boolean toEnd 为true时focus到内容尾部
         * @returns void
         */
        this.focus = function(toEnd) {
            return _instance.focus(toEnd);
        };

        /**
         * ready
         * 编辑器加载成功后执行
         *
         * @param {function} fn 执行的方法
         * @returns void
         */
        this.ready = function(fn) {
            if (typeof fn === 'function') {
                _instance.ready(fn);
            }
        };

        /**
         * sync
         * 同步编辑器内容到form表单
         *
         * @returns void
         */
        this.sync = function() {
            return _instance.sync();
        };

        /**
         * hide
         * 隐藏编辑器
         *
         * @return void
         */
        this.hide = function() {
            return _instance.hide();
        };

        /**
         * show
         * 显示编辑器
         *
         * @return void
         */
        this.show = function() {
            return _instance.show();
        };

        /**
         * disable
         * 禁用编辑器
         *
         * @return void
         */
        this.disable = function() {
            return _instance.disable();
        };

        /**
         * enable
         * 恢复编辑器
         *
         * @return void
         */
        this.enable = function() {
            return _instance.enable();
        };

        /**
         * getDocument
         * 获取编辑器文档对象
         *
         * @return DOM object
         */
        this.getDocument = function() {
            return _instance.document;
        };
    };

    HB.editor = _editor;
})();